using System.Buffers;

using Spectre.Console.Cli;

namespace Geekeey.Extensions.Process.Tests.Dummy.Commands;

internal sealed class EchoStdinCommand : Command<EchoStdinCommand.Settings>
{
	public sealed class Settings : CommandSettings
	{
		[CommandOption("--target")] public OutputTarget Target { get; init; } = OutputTarget.StdOut;
		[CommandOption("--length")] public long Length { get; init; } = long.MaxValue;
	}

	public override int Execute(CommandContext context, Settings settings)
	{
		using var buffer = MemoryPool<char>.Shared.Rent(81920);

		var count = 0L;
		while (count < settings.Length)
		{
			var bytesWanted = (int)Math.Min(buffer.Memory.Length, settings.Length - count);

			var bytesRead = Console.In.Read(buffer.Memory.Span[..bytesWanted]);
			if (bytesRead <= 0)
				break;

			foreach (var writer in settings.Target.GetWriters())
			{
				writer.Write(buffer.Memory.Span[..bytesRead]);
			}

			count += bytesRead;
		}

		return 0;
	}
}