using System.Buffers;
using System.Text;

using Spectre.Console.Cli;

namespace Geekeey.Extensions.Process.Tests.Dummy.Commands;

internal sealed class GenerateBlobCommand : Command<GenerateBlobCommand.Settings>
{
	private readonly Random _random = new(1234567);

	public sealed class Settings : CommandSettings
	{
		[CommandOption("--target")] public OutputTarget Target { get; init; } = OutputTarget.StdOut;
		[CommandOption("--length")] public long Length { get; init; } = 100_000;
		[CommandOption("--buffer")] public int BufferSize { get; init; } = 1024;
	}

	public override int Execute(CommandContext context, Settings settings)
	{
		using var bytes = MemoryPool<byte>.Shared.Rent(settings.BufferSize);
		using var chars = MemoryPool<char>.Shared.Rent(settings.BufferSize);

		var total = 0L;
		while (total < settings.Length)
		{
			_random.NextBytes(bytes.Memory.Span);

			Encoding.UTF8.GetChars(bytes.Memory.Span, chars.Memory.Span);

			var count = (int)Math.Min(chars.Memory.Length, settings.Length - total);
			foreach (var writer in settings.Target.GetWriters())
			{
				writer.Write(chars.Memory[..count]);
			}

			total += count;
		}

		return 0;
	}
}